=begin
(c) TIG 2011-2012
Script:
TIG-arcBYtools.rb
###
Type: TIG::ArcBy.rl
Menu: Draw >> TIG::ArcBy... >> Radius+Length
A dialog asks for Radius, ArcLength and NumberSegments.
The default values are 1m, 1m and 12.
A group containing an arc of that radius and length is added at [0,0,0]
It has clines and cpoints to help relocate it as needed.
The group is named showing the Arc_Radius_Length_Angle
The calculated Angle [to 6dp] might be useful...
If you add arguments after the typed command there is no dialog, e.g.
TIG::ArcBy.rl(1.m,1.m,24)
allowing it to be called from other methods invisibly.
It returns [group,angle] or nil on a failure.
###
Type: TIG::ArcBy.al
Menu: Draw >> TIG::ArcBy... >> SweptAngle+Length
A dialog asks for SweptAngle, ArcLength and NumberSegments.
The default values are 90 degrees, 1m and 12.
A group containing an arc of that swept-angle and length is added at 
[0,0,0]
It has clines and cpoints to help relocate it as needed.
The group is named showing the Arc_Radius_Length_Angle
The calculated Radius [shown to current unit's accuracy] might be useful...
If you add arguments after the typed command there is no dialog, e.g.
TIG::ArcBy.al(45,1.m,18)
allowing it to be called from other methods invisibly.
It returns the [group,radius] or nil on a failure.
###
Type: TIG::ArcBy.rc
Menu: Draw >> TIG::ArcBy... >> Radius+ChordLength
A dialog asks for Radius, ChordLength and NumberSegments.
The default values are 1m, 1m and 12.
A group containing an arc of that radius and chord-length is added at 
[0,0,0]
It has clines and cpoints to help relocate it as needed.
The group is named showing the Arc_Radius_ChordLength_Angle
The calculated Radius [shown to current unit's accuracy] might be useful...
If you add arguments after the typed command there is no dialog, e.g.
TIG::ArcBy.rc(2.m,1.m,18)
allowing it to be called from other methods invisibly.
It returns the [group,angle] or nil on a failure.
It will return nil / error if the ChordLength > Radius*2
###
Type: TIG::ArcBy.rs
Menu: Draw >> TIG::ArcBy... >> Radius+SegmentLength
A dialog asks for Radius, SegmentLength and NumberSegments.
The default values are 1m, 0.1m and 12.
A group containing an arc of that radius with the segment-length and number 
and a claculated swept-angle is added at 
[0,0,0]
It has clines and cpoints to help relocate it as needed.
The group is named showing the Arc_Radius_SegmentLength_Angle
If you add arguments after the typed command there is no dialog, e.g.
TIG::ArcBy.cs(1.m,250.mm,6)
allowing it to be called from other methods invisibly.
It returns the [group,angle] or nil on a failure.
###
Type: TIG::ArcBy.as
Menu: Draw >> TIG::ArcBy... >> SweptAngle+SegmentLength
A dialog asks for SweptAngle, SegmentLength and NumberSegments.
The default values are 90, 0.1m and 12.
A group containing an arc of a calculated radius with the given 
swept-angle, segment-length and number of segments is added at 
[0,0,0]
It has clines and cpoints to help relocate it as needed.
The group is named showing the Arc_Radius_SegmentLength_Angle
The calculated Radius [shown to current unit's accuracy] might be useful...
If you add arguments after the typed command there is no dialog, e.g.
TIG::ArcBy.as(45.0,250.mm,6)
allowing it to be called from other methods invisibly.
It returns the [group,angle] or nil on a failure.
###
Type: TIG::ArcBy.cs
Menu: Draw >> TIG::ArcBy... >> ChordX+ChordY+SegmentLength
A dialog asks for ChordX, ChordY, SegmentLength and NumberSegments.
The default values are 1m, 1m, 0.125m and 12.
A group containing an arc of a calculated radius with the chord from the 
given chord-X/Y offsets, segment-length and number of segments is added at 
[0,0,0]
It has clines and cpoints to help relocate it as needed.
The group is named showing the Arc_Radius_ChordLength_SegmentLength_Angle
The calculated Radius [shown to current unit's accuracy] might be useful...
If you add arguments after the typed command there is no dialog, e.g.
TIG::ArcBy.cs(1.m,500.mm,250.mm,6)
allowing it to be called from other methods invisibly.
It returns the [group,angle] or nil on a failure.
The arc's segment length should be to with ~0.02% of the specified length.
If you can't have an Arc with the given parameters you are told.
###
Donations: by PayPal.com to info @ revitrev.org
###
Version:
1.0 20111111 First issue.
1.1 20120219 Formating of group name adjusted.
1.2 20120412 Menu added.  Extra ArcBy functions added.
=end
require 'sketchup.rb'
###
module TIG;end
###
module TIG::ArcBy
### menu
unless file_loaded?(File.basename(__FILE__))
  submenu=UI.menu('Draw').add_submenu('TIG::ArcBy...')
  submenu.add_item('Radius+Length'){self.rl()}
  submenu.add_item('SweptAngle+Length'){self.al()}
  submenu.add_item('Radius+ChordLength'){self.rc()}
  submenu.add_item('Radius+SegmentLength'){self.rs()}
  submenu.add_item('SweptAngle+SegmentLength'){self.as()}
  submenu.add_item('ChordX+ChordY+SegmentLength'){self.cs()}
end
file_loaded(File.basename(__FILE__))
###
def self.rl(r=nil,l=nil,n=nil)
  t='TIG::ArcBy: Radius+Length'
  inNOTnil=r
  if not r
    results=inputbox(['Radius: ','ArcLength: ','NumberSegments: '],[1.m, 1.m, 12],t)
    return nil if not results
    r,l,n=results
  end
  return nil unless r and l and n
  cir=2*r*Math::PI
  if l > cir
    unless inNOTnil
      UI.messagebox("#{t}\n\nArcLength cannot be > 2*PI*r !\n\nArcLength = #{l.to_f.to_l}\n2*PI*#{r.to_f.to_l} = #{cir.to_f.to_l}")
      self.rl()
      return nil
    end
    puts "ArcLength cannot be > 2*PI*r"
    return nil
  end
  if l <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nArcLength [#{l.to_f.to_l}] cannot be <= 0")
      self.rl()
      return nil
    end
    puts "ArcLength cannot be <= 0"
    return nil
  end
  if r <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nRadius [#{r.to_f.to_l}] cannot be <= 0")
      self.rl()
      return nil
    end
    puts "Radius cannot be <= 0"
    return nil
  end
  if n < 2
    unless inNOTnil
      UI.messagebox("#{t}\n\nNumberSegments [#{n}] cannot be < 2")
      self.rl()
      return nil
    end
    puts "NumberSegments cannot be < 2"
    return nil
  end
  p0=[0,0,0]
  p1=[r,0,0]
  a=360.degrees*l/(2*r*Math::PI)
  p2=p1.transform(Geom::Transformation.rotation(p0,[0,0,1],a))
  m=Sketchup.active_model
  m.start_operation(t)
  g=m.entities.add_group()
  g.name='Arc_R'+r.to_s+'_L'+l.to_s+'_A'+(sprintf("%.6f",a.radians))
  ge=g.entities
  ge.add_cpoint(p0)
  ge.add_cpoint(p1)
  ge.add_cpoint(p2)
  ge.add_cline(p0,p1)
  ge.add_cline(p0,p2)
  ge.add_cline(p1,p2)
  ge.add_arc(p0,[1,0,0],[0,0,1],r,0,a,n)
  m.commit_operation
  return [g, a.radians]
end
###
def self.al(a=nil,l=nil,n=nil)
  t='TIG::ArcBy: SweptAngle+Length'
  inNOTnil=a
  if not a
    results=inputbox(['SweptAngle: ','ArcLength: ','NumberSegments: '],[90.0, 1.m, 12],t)
    return nil if not results
    a,l,n=results
  end
  return nil unless a and l and n
  if a==0 or a.abs > 360
    unless inNOTnil
      UI.messagebox("#{t}\n\nSweptAngle [#{a}] cannot be 0 or > 360 degrees or < -360 degrees")
      self.al()
      return nil
    end
    puts "SweptAngle cannot be 0 or > 360 degrees or < -360 degrees"
    return nil
  end
  if l <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nArcLength [#{l.to_f.to_l}] cannot be <= 0")
      self.al()
      return nil
    end
    puts "ArcLength cannot be <= 0"
    return nil
  end
  if n < 2
    unless inNOTnil
      UI.messagebox("#{t}\n\nNumberSegments [#{n}] cannot be < 2")
      self.al()
      return nil
    end
    puts "NumberSegments cannot be < 2"
    return nil
  end
  r=(360.degrees*l/(a.degrees*2*Math::PI))
  p0=[0,0,0]
  p1=[r,0,0]
  p2=p1.transform(Geom::Transformation.rotation(p0,[0,0,1],a.degrees))
  m=Sketchup.active_model
  m.start_operation(t)
  g=m.entities.add_group()
  g.name='Arc_R'+(r.to_f.to_l.to_s.gsub(/ /,''))+'_L'+l.to_s+'_A'+a.to_s
  ge=g.entities
  ge.add_cpoint(p0)
  ge.add_cpoint(p1)
  ge.add_cpoint(p2)
  ge.add_cline(p0,p1)
  ge.add_cline(p0,p2)
  ge.add_cline(p1,p2)
  ge.add_arc(p0,[1,0,0],[0,0,1],r,0,a.degrees,n)
  m.commit_operation
  return [g, r]
end
###
def self.rc(r=nil,c=nil,n=nil)
  t='TIG::ArcBy: Radius+ChordLength'
  inNOTnil=r
  if not r
    results=inputbox(['Radius: ','ChordLength: ','NumberSegments: '],[1.m, 1.m, 12],t)
    return nil if not results
    r,c,n=results
  end
  return nil unless r and c and n
  if c > r*2
    unless inNOTnil
      UI.messagebox("#{t}\n\nChordLength cannot be > Radius*2 !\n\nRadius is #{r.to_f.to_l}\nChordLength is #{c.to_f.to_l}")
      self.rc()
      return nil
    end
    puts "ChordLength cannot be > Radius*2"
    return nil
  end
  if c <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nChordLength [#{c.to_f.to_l}] cannot be <= 0")
      self.rc()
      return nil
    end
    puts "nChordLength cannot be <= 0"
    return nil
  end
  if r <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nRadius [#{r.to_f.to_l}] cannot be <= 0")
      self.rc()
      return nil
    end
    puts "Radius cannot be <= 0"
    return nil
  end
  if n < 2
    unless inNOTnil
      UI.messagebox("#{t}\n\nNumberSegments [#{n}] cannot be < 2")
      self.rc()
      return nil
    end
    puts "NumberSegments cannot be < 2"
    return nil
  end
  p0=[0,0,0]
  p1=[r,0,0]
  a=2*(Math::asin((c/2)/r))
  p2=p1.transform(Geom::Transformation.rotation(p0,[0,0,1],a))
  m=Sketchup.active_model
  m.start_operation(t)
  g=m.entities.add_group()
  g.name='Arc_R'+r.to_s+'_C'+c.to_s+'_A'+(sprintf("%.6f",a.radians))
  ge=g.entities
  ge.add_cpoint(p0)
  ge.add_cpoint(p1)
  ge.add_cpoint(p2)
  ge.add_cline(p0,p1)
  ge.add_cline(p0,p2)
  ge.add_cline(p1,p2)
  ge.add_arc(p0,[1,0,0],[0,0,1],r,0,a,n)
  m.commit_operation
  return [g, a.radians]
end
###
###
def self.rs(r=nil,s=nil,n=nil)
  t='TIG::ArcBy: Radius+SegmentLength'
  inNOTnil=r
  if not r
    results=inputbox(['Radius: ','SegmentLength: ','NumberSegments: '],[1.m, 0.1.m, 12],t)
    return nil if not results
    r,s,n=results
  end
  return nil unless r and r and n
  cir=2*r*Math::PI
  if cir <= s*n
    unless inNOTnil
      UI.messagebox("#{t}\n\nSegmentLength*NumberSegments cannot be >= 2*PI*r\n#{s.to_f.to_l} * #{n} = #{(n*s).to_f.to_l}\n2*PI*#{r.to_f.to_l} = #{cir.to_f.to_l}")
      self.rs()
      return nil
    end
    puts "SegmentLength*NumberSegments cannot be > 2*PI*r"
    return nil
  end
  if s <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nSegmentLength [#{s.to_f.to_l}] cannot be <= 0")
      self.rs()
      return nil
    end
    puts "nSegmentLength cannot be <= 0"
    return nil
  end
  if r <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nRadius [#{r.to_f.to_l}] cannot be <= 0")
      self.rs()
      return nil
    end
    puts "Radius cannot be <= 0"
    return nil
  end
  if n < 2
    unless inNOTnil
      UI.messagebox("#{t}\n\nNumberSegments [#{n}] cannot be < 2")
      self.rs()
      return nil
    end
    puts "NumberSegments cannot be < 2"
    return nil
  end
  p0=[0,0,0]
  p1=[r,0,0]
  a=2*n*Math::asin(s/2/r)
  p2=p1.transform(Geom::Transformation.rotation(p0,[0,0,1],a))
  m=Sketchup.active_model
  m.start_operation(t)
  g=m.entities.add_group()
  g.name='Arc_R'+r.to_s+'_S'+s.to_s+'_A'+(sprintf("%.6f",a.radians))
  ge=g.entities
  ge.add_cpoint(p0)
  ge.add_cpoint(p1)
  ge.add_cpoint(p2)
  ge.add_cline(p0,p1)
  ge.add_cline(p0,p2)
  ge.add_cline(p1,p2)
  ge.add_arc(p0,[1,0,0],[0,0,1],r,0,a,n)
  m.commit_operation
  return [g, a.radians]
end
###
def self.as(a=nil,s=nil,n=nil)
  t='TIG::ArcBy: SweptAngle+SegmentLength'
  inNOTnil=a
  if not a
    results=inputbox(['SweptAngle: ','SegmentLength: ','NumberSegments: '],[90.0, 0.1.m, 12],t)
    return nil if not results
    a,s,n=results
  end
  return nil unless a and s and n
  if a==0 or a.abs > 360
    unless inNOTnil
      UI.messagebox("#{t}\n\nSweptAngle [#{a}] cannot be 0 or > 360 degrees or < -360 degrees")
      self.as()
      return nil
    end
    puts "SweptAngle cannot be 0 or > 360 degrees or < -360 degrees"
    return nil
  end
  if s <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nSegmentLength [#{s.to_f.to_l}] cannot be <= 0")
      self.as()
      return nil
    end
    puts "nSegmentLength cannot be <= 0"
    return nil
  end
  if n < 2
    unless inNOTnil
      UI.messagebox("#{t}\n\nNumberSegments [#{n}] cannot be < 2")
      self.as()
      return nil
    end
    puts "NumberSegments cannot be < 2"
    return nil
  end
  r=s/2/Math::sin(a.degrees/n/2)
  p0=[0,0,0]
  p1=[r,0,0]
  p2=p1.transform(Geom::Transformation.rotation(p0,[0,0,1],a.degrees))
  m=Sketchup.active_model
  m.start_operation(t)
  g=m.entities.add_group()
  g.name='Arc_R'+(r.to_f.to_l.to_s.gsub(/ /,''))+'_S'+s.to_s+'_A'+a.to_s
  ge=g.entities
  ge.add_cpoint(p0)
  ge.add_cpoint(p1)
  ge.add_cpoint(p2)
  ge.add_cline(p0,p1)
  ge.add_cline(p0,p2)
  ge.add_cline(p1,p2)
  ge.add_arc(p0,[1,0,0],[0,0,1],r,0,a.degrees,n)
  m.commit_operation
  return [g, r]
end
###
def self.cs(cx=nil,s=nil,n=nil)
  t='TIG::ArcBy: ChordX+ChordY+SegmentLength'
  inNOTnil=cx
  if not cx
    results=inputbox(['ChordX: ','ChordY: ','SegmentLength: ','NumberSegments: '],[1.m,1.m, 0.125.m, 12],t)
    return nil if not results
    cx,cy,s,n=results
  end
  return nil unless cx and cy and s and n
  if cx == 0 and cy == 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nChordX and ChordY [#{cx.to_f.to_l} / #{cy.to_f.to_l}] cannot be 0")
      self.cs()
      return nil
    end
    puts "ChordX and ChordY cannot be 0"
    return nil
  end
  if cx < 0 or cy < 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nChordX or ChordY [#{cx.to_f.to_l} / #{cy.to_f.to_l}] cannot be < 0")
      self.cs()
      return nil
    end
    puts "ChordX or ChordY cannot be < 0"
    return nil
  end
  ### calc chord length
  c=Math::sqrt((cx*cx)+(cy*cy))
  ###
  if s <= 0
    unless inNOTnil
      UI.messagebox("#{t}\n\nSegmentLength [#{s.to_f.to_l}] cannot be <= 0")
      self.cs()
      return nil
    end
    puts "SegmentLength cannot be <= 0"
    return nil
  end
  if n < 2
    unless inNOTnil
      UI.messagebox("#{t}\n\nNumberSegments [#{n}] cannot be < 2")
      self.cs()
      return nil
    end
    puts "NumberSegments cannot be < 2"
    return nil
  end
  if s*n <= c
    unless inNOTnil
      UI.messagebox("#{t}\n\nSegmentLength [#{s.to_f.to_l}] * NumberSegments [#{n}] = #{(n*s).to_f.to_l}\ncannot be <= ChordLength [#{c.to_f.to_l}]")
      self.cs()
      return nil
    end
    puts "SegmentLength*NumberSegments cannot be <= ChordLength"
    return nil
  end
  ### make approximation of arc
  l = s*n
  r= 0.01.mm
  r+=10000.mm while 2.0*r*(Math::sin(l/2.0/r)) < c
  r-=1000.mm while 2.0*r*(Math::sin(l/2.0/r)) > c
  r+=100.mm while 2.0*r*(Math::sin(l/2.0/r)) < c
  r-=10.mm while 2.0*r*(Math::sin(l/2.0/r)) > c
  r+=1.mm while 2.0*r*(Math::sin(l/2.0/r)) < c
  r-=0.1.mm while 2.0*r*(Math::sin(l/2.0/r)) > c
  r+=0.01.mm while 2.0*r*(Math::sin(l/2.0/r)) < c
  r-=0.001.mm while 2.0*r*(Math::sin(l/2.0/r)) > c
  r+=0.0001.mm while 2.0*r*(Math::sin(l/2.0/r)) < c
  r-=0.00001.mm while 2.0*r*(Math::sin(l/2.0/r)) > c
  r+=0.000001.mm while 2.0*r*(Math::sin(l/2.0/r)) < c
  r-=0.0000001.mm while 2.0*r*(Math::sin(l/2.0/r)) > c
  ###
  a=2*(Math::asin((c/2.0)/r))
  ###
  m=Sketchup.active_model
  m.start_operation(t)
  g=m.entities.add_group()
  ge=g.entities
  ### fine tune r and a to s
  edges=ge.add_arc(ORIGIN,X_AXIS,Z_AXIS,r,0,a,n)
  seg=edges[0].length
  if seg > s
    while seg > s
      begin
        r+=0.1.mm
        a=2*(Math::asin((c/2)/r))
        ge.erase_entities(ge.to_a)
        edges=ge.add_arc(ORIGIN,X_AXIS,Z_AXIS,r,0,a,n)
        seg=edges[0].length
      rescue
        UI.messagebox("#{t}\n\nIt is not possible to make an Arc to all of these parameters...")
        m.abort_operation
        self.cs()
        return nil
      end
    end
  end
  if seg < s
    while seg < s
      begin
        r-=0.1.mm
        a=2*(Math::asin((c/2)/r))
        ge.erase_entities(ge.to_a)
        edges=ge.add_arc(ORIGIN,X_AXIS,Z_AXIS,r,0,a,n)
        seg=edges[0].length
      rescue
        UI.messagebox("#{t}\n\nIt is not possible to make an Arc to all of these parameters...")
        m.abort_operation
        self.cs()
        return nil
      end
    end
  end
  ###
  ge.erase_entities(ge.to_a)
  ###
  pm=Geom::Point3d.new(cx/2,cy/2,0)
  ve=[cx,0,0].vector_to([0,cy,0])
  ve.length=Math::sqrt((r*r)-((c/2)*(c/2)))
  tr=Geom::Transformation.rotation(pm,[0,0,1],90.degrees)
  ve.transform!(tr)
  ###
  p0=pm.offset(ve)
  p1=[cx,0,0]
  p2=p1.transform(Geom::Transformation.rotation(p0,[0,0,1],a))
  ###
  g.name='Arc_R'+r.to_f.to_l.to_s.gsub(/ /,'')+'_C'+c.to_f.to_l.to_s.gsub(/ /,'')+'_S'+s.to_s+'_A'+(sprintf("%.6f",a.radians))
  ge.add_arc(p0, (p0.vector_to([cx,0,0])), [0,0,1], r,0,a,n)
  ge.add_cpoint(p0)
  ge.add_cline(p0,[cx,0,0])
  ge.add_cline(p0,[0,cy,0])
  ge.add_cline(p0,pm)
  ge.add_cpoint(pm)
  ge.add_cpoint([0,0,0])
  ge.add_cpoint([cx,0,0])
  ge.add_cpoint([0,cy,0])
  ge.add_cpoint([cx,cy,0])
  ge.add_cline([0,0,0],[cx,0,0])
  ge.add_cline([0,0,0],[0,cy,0])
  ge.add_cline([cx,0,0],[0,cy,0])
  ge.add_cline([0,cy,0],[cx,cy,0])
  ge.add_cline([cx,cy,0],[cx,0,0])
  ge.add_cline([cx,cy,0],[0,0,0])
  tr=Geom::Transformation.translation(p0.vector_to(ORIGIN))
  g.transform!(tr)
  m.commit_operation
  return [g, a.radians]
end
###
end#module
###


